Type: Package
Package: PSAboot
Title: Bootstrapping for Propensity Score Analysis
Version: 1.3.9
Date: 2025-10-22
Authors@R: 
    person("Jason", "Bryer", , "jason@bryer.org", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-2454-0402"))
Maintainer: Jason Bryer <jason@bryer.org>
Description: It is often advantageous to test a hypothesis more than once
    in the context of propensity score analysis (Rosenbaum, 2012)
    <doi:10.1093/biomet/ass032>. The functions in this package facilitate
    bootstrapping for propensity score analysis (PSA). By default,
    bootstrapping using two classification tree methods (using 'rpart' and
    'ctree' functions), two matching methods (using 'Matching' and
    'MatchIt' packages), and stratification with logistic regression.  A
    framework is described for users to implement additional propensity
    score methods.  Visualizations are emphasized for diagnosing balance;
    exploring the correlation relationships between bootstrap samples and
    methods; and to summarize results.
License: GPL
URL: https://github.com/jbryer/PSAboot
BugReports: https://github.com/jbryer/PSAboot/issues
Depends: ggplot2, graphics, PSAgraphics, R (>= 4.1.0)
Imports: ggthemes, Matching, MatchIt, modeltools, parallel, party,
        psych, reshape2, rpart, stats, TriMatch, utils
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Encoding: UTF-8
LazyLoad: yes
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-10-22 14:46:43 UTC; jasonbryer
Author: Jason Bryer [aut, cre] (ORCID: <https://orcid.org/0000-0002-2454-0402>)
Repository: CRAN
Date/Publication: 2025-10-22 23:30:16 UTC
Built: R 4.6.0; ; 2026-05-01 02:17:38 UTC; unix
