# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,PSAbootSummary)
S3method(boxplot,PSAboot)
S3method(boxplot,PSAboot.balance)
S3method(hist,PSAboot)
S3method(plot,PSAboot)
S3method(plot,PSAboot.balance)
S3method(print,PSAboot)
S3method(print,PSAboot.balance)
S3method(print,PSAbootSummary)
S3method(summary,PSAboot)
export(PSAboot)
export(balance)
export(balance.matching)
export(boot.ctree)
export(boot.matching)
export(boot.matchit)
export(boot.rpart)
export(boot.strata)
export(boot.weighting)
export(getPSAbootMethods)
export(matrixplot)
export(psa.strata)
export(q25)
export(q75)
import(MatchIt)
import(Matching)
import(PSAgraphics)
import(TriMatch)
import(ggplot2)
import(graphics)
import(modeltools)
import(parallel)
import(reshape2)
import(rpart)
import(utils)
importFrom(ggthemes,geom_tufteboxplot)
importFrom(party,ctree)
importFrom(party,where)
importFrom(psych,describe)
importFrom(psych,describeBy)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,fitted)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,quasibinomial)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(stats,update.formula)
importFrom(stats,var)
importFrom(stats,weighted.mean)
